//
//  PostSemanticSearch.swift
//  SampleApp
//
//  Created by Nishan on 7/8/2567 BE.
//  Copyright © 2567 BE David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct PostSemanticSearchView: View {
    
    @State private var targetId: String = ""
    @State private var targetType: Int = 2 // 2 means any
    @State private var query: String = ""
    @State private var matchParentPost: Bool = false
    @State private var dataTypes: Int = 2 // 0: Text, 1: Image, 2: Both
    @State private var tags: String = ""
    @State private var showSearchSheet = false
    
    @StateObject var viewModel: PostSemanticSearchViewModel = PostSemanticSearchViewModel()
    
    var body: some View {
        VStack {
            
            List {
                ForEach(viewModel.searchedPosts, id: \.postId) { post in
                    VStack(alignment: .leading) {
                        Text(post.modelDescription)
                        
                        if !post.childrenPosts.isEmpty {
                            Text("Children Posts:")
                            ForEach(post.childrenPosts, id: \.postId) { childPost in
                                Text("""
                                    - Post Id: \(childPost.postId)
                                    - Type: \(childPost.dataType)
                                    - FileId: \(childPost.getImageInfo()?.fileId ?? "")
                                    - FileURL: \(childPost.getImageInfo()?.fileURL ?? "")
                                    ---
                                    """
                                )
                                .font(.subheadline)
                                .fontWeight(.regular)
                                .foregroundColor(Color(UIColor.label))
                            }
                        }
                    }
                    .font(.body)
                    .foregroundColor(Color(UIColor.label))

                }
            }
            
            AmityDefaultButton(title: "Search Posts") {
                showSearchSheet.toggle()
            }
            .alert("", isPresented: $viewModel.showAlert) {
                Button {
                    viewModel.showAlert = false
                } label: {
                    Text("Dismiss")
                }
            } message: {
                Text(viewModel.alertMessage)
            }
            
        }.sheet(isPresented: $showSearchSheet, content: {
            NavigationView {
                ScrollView {
                    VStack(alignment: .leading, spacing: 12) {
                        
                        Text("Search Query:")
                            .asFormLabel()
                            .padding(.top, 16)
                        
                        TextField("Enter Search Query", text: $query)
                            .modifier(AmityTextFieldStyle())
                        
                        Text("Target Id (optional):")
                            .asFormLabel()
                        
                        TextField("Enter Target Id:", text: $targetId)
                            .modifier(AmityTextFieldStyle())
                        
                        Text("Target Type:")
                            .asFormLabel()

                        Picker("Target Type", selection: $targetType) {
                            Text("User")
                                .tag(0)
                            Text("Community")
                                .tag(1)
                            Text("Any")
                                .tag(2)
                        }
                        .pickerStyle(.segmented)
                        
                        Text("Data Types:")
                            .asFormLabel()

                        Picker("Data Types", selection: $dataTypes) {
                            Text("Text")
                                .tag(0)
                            Text("Image")
                                .tag(1)
                            Text("Both")
                                .tag(2)
                        }
                        .pickerStyle(.segmented)
                                        
                        Toggle("Match Parent Posts", isOn: $matchParentPost)
                            .padding(.vertical, 8)
                    }
                    .padding(.horizontal)
                }
                .navigationTitle("Search Options")
                .navigationBarTitleDisplayMode(.inline)
                .toolbar(content: {
                    ToolbarItem(placement: .primaryAction) {
                        Button(action: {
                            
                            let postTargetType: AmityPostTargetType? = AmityPostTargetType(rawValue: targetType) ?? nil
                            let postTargetId: String? = targetId.isEmpty ? nil : targetId
                            var postDataTypes: [String] = []
                            
                            if dataTypes == 2 {
                                postDataTypes = ["text", "image"]
                            } else if dataTypes == 1 {
                                postDataTypes = ["image"]
                            } else {
                                postDataTypes = ["text"]
                            }
                            
                            let searchOptions = AmityPostSemanticSearchOptions.init(query: query, targetId: postTargetId, targetType: postTargetType, dataTypes: postDataTypes, matchingOnlyParentPost: matchParentPost)
                            viewModel.searchPosts(options: searchOptions)
                            
                            showSearchSheet = false
                        }, label: {
                            Text("Search")
                        })
                    }
                })
            }
            .modifier(HalfModal())
        })
    }
    
    struct HalfModal: ViewModifier {
        
        func body(content: Content) -> some View {
            if #available(iOS 16.0, *) {
                content
                    .presentationDetents([.fraction(0.68), .large])
            } else {
                content
            }
        }
    }
}

#Preview {
    PostSemanticSearchView()
}

class PostSemanticSearchViewModel: ObservableObject {
    
    @Published var searchedPosts: [AmityPost] = []
    @Published var showAlert = false
    @Published var alertMessage = ""
    
    let postRepo = AmityPostRepository(client: AmityManager.shared.client!)
    var token: AmityNotificationToken?
    
    func searchPosts(options: AmityPostSemanticSearchOptions) {
        token?.invalidate()
        token = nil
        
        searchedPosts = []
        
        token = postRepo.semanticSearchPosts(options: options).observe { liveCollection, _, error in
            
            if let error {
                self.showAlert = true
                self.alertMessage = error.localizedDescription
                
                self.searchedPosts = []
                return
            }
            
            self.searchedPosts = liveCollection.allObjects()
        }
    }
}
